#!/usr/bin/env bash

# Usage: ./exp.sh

mf=machinefile
data_dir=./data
root_dir=$PWD
code_dir=./solvers
log_dir=./logs
model_dir=./models
nodes=$(wc ${mf} | awk '{print $1}')
cmd_mpirun="mpirun -n ${nodes} -npernode 1 --machinefile ${mf} -x OMP_NUM_THREADS=1"
eps=8e-5

# Uncomment lines to include more data sets
data_list=(
"rcv1_train.binary 16"
#"real-sim 16"
#"news20.binary 1024"
#"rcv1_test.binary 4"
#"url_combined 8"
#"webspam_wc_normalized_trigram.svm 64"
#"kddb 0.5"
#"avazu-site 1"
#"kdd12.svm 2"
#"epsilon_normalized 0.5"
)

while IFS= read line
do
    if [ "${line}" != "localhost" ]; then
        echo "syncing to ${line}..."
        rsync -e "ssh -o StrictHostKeyChecking=no" -a ${root_dir}/${code_dir} ${line}:${root_dir}
    fi
done < ${mf}

${cmd_mpirun} make -B -C ${code_dir}
${cmd_mpirun} mkdir -p ${log_dir} ${model_dir} ${data_dir}

for i in "${data_list[@]}"; do
    data_c=(${i})
    data=${data_c[0]}
    c=${data_c[1]}

    if [ ! -f ${data_dir}/${data} ]; then
        wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary/${data}.bz2
        bzip2 -d ${data}.bz2
        mv ${data} ${data_dir}
    fi
    ./data/split.py ${mf} ${data_dir}/${data} ${data_dir}/${data}
    #row-format
    for s in 21 33; do
        log_name="compare_${data}_s${s}.txt"
        cmd_train="${code_dir}/train -e ${eps} -c ${c} -s ${s} ${data_dir}/${data} ${model_dir}/${data}.model"
        echo ${cmd_train} | tee ${log_dir}/${log_name}
        ${cmd_mpirun} ${cmd_train} | tee -a ${log_dir}/${log_name}
    done
done

