#ifndef _TRON_H
#define _TRON_H

class function
{
public:
	virtual double fun(double *w) = 0 ;
	virtual void grad(double *w, double *g) = 0 ;
	virtual void Hv(double *s, double *Hs) = 0 ;

	virtual int get_nr_variable(void) = 0 ;
	virtual ~function(void){}
};

class TRON
{
public:
	TRON(const function *fun_obj, double eps = 0.1, double eps_cg = 0.1, int max_iter = 1000);
	~TRON();

	void tron(double *w);
	void set_print_string(void (*i_print) (const char *buf));

private:
	int trcg(double delta, double *g, double *s, double *r, bool *reach_boundary);
	double norm_inf(int n, double *x);

	double eps;
	double eps_cg;
	int max_iter;
	function *fun_obj;
	void info(const char *fmt,...);
	void (*tron_print_string)(const char *buf);
};
#endif

class LBFGS
{
public:
	LBFGS(const function *fun_obj, double eps = 0.1, int m=10, double eta = 1e-2, int max_iter = 2000);
	~LBFGS();

	void lbfgs(double *w);
	void set_print_string(void (*i_print) (const char *buf));

private:
	void TwoLoop(double* g, double** s, double** y, double* rho, int DynamicM, double *step, int k, int n);
	double eps;
	double eta;
	int max_iter;
	int M;
	function *fun_obj;
	void info(const char *fmt,...);
	void (*lbfgs_print_string)(const char *buf);
};
