import sys
import matplotlib
matplotlib.use('agg')
import matplotlib.pyplot as plt
import re
import os
import itertools

DATA_DIR = 'logs'
OPT_DIR = '../opt_logs'
FIG_DIR = 'figs'
SAVE_FIG = True

# Uncomment lines to specify the data sets to be plotted
DATASETS = (
    'heart_scale',
    #'rcv1_test.binary',
    #'real-sim',
    #'news20.binary',
    #'url_combined',
)

data_ratio = {}
with open('../stat.txt') as f:
    for line in f:
        data, l, n, nnz, lg_c, pos, neg = line.strip().split()
        ratio = 1.0 * min(int(pos), int(neg)) / int(l)
        data_ratio[data] = ratio

stop_cond_std_solver = 'owlqn'
stop_cond_eps_list = (1e-2, 1e-3, 1e-4)
stop_cond_ls_list = ('-', '--', '-.', ':')

data_filenames = os.listdir(DATA_DIR)
opt_filenames = os.listdir(OPT_DIR)
DATA_FILENAME_FORMAT = '{}.{}.{}.{}.{}.log'

OPT_FILENAME_FORMAT = '{}.origin.{}.1e-07.10.log'

C_LIST = (0.1, 1, 10)
E_LIST = (0.00005,)
M_LIST = (21,)

SOLVER_LIST = (
    'owlqn',
    'owlcomm-xp',
    'owlcomm',
    'newton',
)

LEGEND_DIC = {
    "owlqn": "OWLQN",
    'owlcomm-xp': "LCOMM",
    'owlcomm': "LCOMM-approx",
    'newton': "NEWTON",
}

AXIS_DIC = {
    "iter": "Iteration",
    "nnz_ops": "Data pass",
    "time": "Training time (s)",
}

# starting iteration
START = 0

X_AXES = (
    'iter',
    # 'total_fun_eval',
    #'nnz_ops',
    #'time',
)

# True/False: y-axis is relative/absolute function value
RELATIVE_VALUE = True

dataset2obj = {
}

COLORS = (
    'red',
    'orange',
    'green',
    #'cyan',
    'blue',
    'purple',
    'black',
)

LINESTYLES = (
    '-',
    '-.',
    ':',
    '--',
)

MARKERS = (
    'X',
    'D',
    'o',
    '^',
)

NUMOFMARKERS = 5
BIGGER_SIZE = 16
MARKER_SIZE = 8

for figure_i, dataset in enumerate(DATASETS):
    # plot one figure
    plt.rc('axes', labelsize=BIGGER_SIZE)
    plt.rc('legend', fontsize=BIGGER_SIZE)
    plt.rc('xtick', labelsize=15)
    plt.rc('ytick', labelsize=15)

    figure_settings = list(itertools.product(C_LIST, E_LIST))
    num_settings = len(figure_settings)

    fig_size = 5
    x_axes_horizontal = True
    if x_axes_horizontal:
        fig = plt.figure(figure_i + 1, figsize=(fig_size*len(X_AXES), fig_size*num_settings))
    else:
        fig = plt.figure(figure_i + 1, figsize=(fig_size*num_settings, fig_size*len(X_AXES)))

    fig.canvas.set_window_title(dataset)
    fig.set_tight_layout(True)
    legends = []

    for subplot_i, (x_axis, c_e) in enumerate(itertools.product(X_AXES, figure_settings)):
        # plot one subplot

        if x_axes_horizontal:
            plt.subplot(num_settings, len(X_AXES), subplot_i + 1)
        else:
            plt.subplot(len(X_AXES), num_settings, subplot_i + 1)

        c, e = c_e
        if RELATIVE_VALUE and dataset in dataset2obj:
            obj = dataset2obj[dataset]
        elif RELATIVE_VALUE:
            opt_filename = OPT_FILENAME_FORMAT.format(dataset, c)
            if not opt_filename in opt_filenames:
                print('{} does not exist, exiting...'.format(opt_filename))
                exit(1)

            opt_filepath = os.path.join(OPT_DIR, opt_filename)
            with open(opt_filepath) as f:
                for line in f:
                    if line.startswith('Objective'):
                        match = re.search(r' = (?P<obj>[\d.eE+-]+)', line)
                        obj = float(match.group('obj'))
        
        xmin = float('INF')
        xmax = 0
        stop_cond_done = 0
        subplot_settings = list(itertools.product(M_LIST, SOLVER_LIST))
        for curve_i, (m, s) in enumerate(subplot_settings):
            # plot one curve

            if 'newton' in s:
                m = 50

            config = (dataset, s, c, e, m)
            data_filename = DATA_FILENAME_FORMAT.format(*config)
            if not data_filename in data_filenames:
                print('{} does not exist, exiting...'.format(data_filename))
                exit(1)
            
            data_filepath = os.path.join(DATA_DIR, data_filename)
            keys = []
            values = []
            with open(data_filepath) as f:
                for line_i, line in enumerate(f):
                    if line.startswith('iter=') or line.startswith('iter ='):
                        pattern = r'{}[ =]+(?P<key>[\d.eE+-]+)'.format(x_axis)
                        match = re.search(pattern, line)
                        key = float(match.group('key'))
                        keys.append(key)
                        
                        pattern = r'f[ =]+(?P<value>[\d.eE+-]+)'
                        match = re.search(pattern, line)
                        value = float(match.group('value'))
                        if RELATIVE_VALUE:
                            value = (value - obj) / obj
                        values.append(value)

                        if s == stop_cond_std_solver:
                            pattern = r'\|pg\|[ =]+(?P<pgnorm>[\d.eE+-]+)'
                            match = re.search(pattern, line)
                            pgnorm = float(match.group('pgnorm'))
                            if line_i == 0:
                                pgnorm0 = pgnorm
                            for stop_cond_eps, stop_cond_ls in list(zip(stop_cond_eps_list, stop_cond_ls_list))[stop_cond_done:]:
                                if pgnorm / pgnorm0 <= stop_cond_eps * data_ratio[dataset]:
                                    plt.axhline(y=value, color='black', ls=stop_cond_ls)
                                    stop_cond_done += 1
                                    break
            
            # TODO: change if more epsilons or C_LIST are added
            leg, = plt.plot(
                keys[START:],
                values[START:],
                COLORS[SOLVER_LIST.index(s)],
                marker = MARKERS[SOLVER_LIST.index(s)],
                markevery = len(values[START:])//NUMOFMARKERS,
                markersize = MARKER_SIZE,
                # ls=LINESTYLES[SOLVER_LIST.index(s) % 2]
            )
            legends.append(leg)
            if keys:
                # assuming the rightmost of 'keys' is the largest one
                xmax = max(xmax, keys[-1])
                xmin = min(xmin, keys[-1])
        
        plt.legend(legends, ['{}'.format(LEGEND_DIC[s]) for m, s in subplot_settings])
        plt.xlabel(AXIS_DIC[x_axis])
        plt.ylabel("Relative Function Difference" if RELATIVE_VALUE else 'f')
        plt.yscale('log')
        # if xmax != float('INF'):
        #     plt.xlim(xmax=min(xmin * 3, xmax))


        #plt.title('C = {}, eps = {}'.format(c, e))
        #plt.title('C = Best C'.format(c))

    if SAVE_FIG:
        if not os.path.exists(FIG_DIR):
            os.makedirs(FIG_DIR)
        fig_filename = '{}_start{}.png'.format(dataset, START)
        fig_filepath = os.path.join(FIG_DIR, fig_filename)
        fig.savefig(fig_filepath, dpi = fig.dpi)
    
    print('Finish {} ({}/{})'.format(dataset, figure_i + 1, len(DATASETS)))

# plt.show()
