#!/usr/bin/env python3
from multiprocessing import Pool
import os, json, time, getopt, sys

# binary template for solvers
BINARY_PATH = './solvers/train'

# data path template
DATA_DIR = './data/{}'

#run_cmd = PRE+"/lr/{args[tp]}/train -s {args[s]} -c {args[c]} {args[fpath]} > {log}"
run_cmd = BINARY_PATH + " -s {args[s]} -c {args[c]} -e {args[e]} -m {args[m]} {args[fpath]} {model_dir}{args[fname]}.model > {log}"
run_multi_cmd = BINARY_PATH + " -s {args[s]} -n 8 -c {args[c]} -e {args[e]} {args[fpath]} > {log}"

# use best_c or not
USE_BEST = True
if USE_BEST:
    cs = [0.1, 1, 10]
else:
    cs = [1, 10, 100, 1000]
LOG_P = './logs/'
MODEL_P = './models/'
LOG_DIR = LOG_P + '{args[fname]}.{args[tp]}.{args[c_name]}.{args[e]}.{args[m]}.log'

# eps options

#eps = [1e-4]
eps = [0.00005]
# set eps exception for some data sets
eps_exception = {
        'kdda': 1e-6,
        'kddb': 0.001,
        'kdd12.svm': 1e-5,
        'criteo.trva': 1e-5,
        }
# but solver in eps_ref will use eps
eps_ref = ['tron_simp_r4']

# Uncomment lines to include more data sets
datasets = [
    'heart_scale',
    #'rcv1_test.binary',
    #'real-sim',
    #'news20.binary',
    #'url_combined',
]
if len(datasets) == 0:
    print('Please select some data for experiments !!')
    exit(1)

'''
owlqn:      OWLQN
owlcomm:    LCOMM-approx
owlcomm-xp: LCOMM
newton:     NEWTON
'''

tps = [
    'owlqn',
    'owlcomm',
    'owlcomm-xp',
    'newton',
]

ARG_DIC = {
    'owlqn': (21, 21),
    'owlcomm': (33, 21),
    'owlcomm-xp': (39, 21),
    'newton': (25, 50),
}

bestC_dic = json.load(open('bestC.json'))
best_c = {
    'lr': dict(**bestC_dic['l1lr']),
}

pool_num = 1
run_opt = False
fun_type = 'lr'

def usage():
    print('./run.py [-r] [-n thread_num] [-f funtion_type]')

def opt_parse(opts):
    for opt in opts:
        if opt[0] == '-r':
            global run_opt
            run_opt = True
        elif opt[0] == '-n':
            if opt[1].isdigit():
                global pool_num
                pool_num = int(opt[1])
            else:
                print('Not an integer !!')
                exit(2)
        elif opt[0] == '-f':
            if opt[1] in ('l2', 'lr'):
                global fun_type
                fun_type = opt[1]
            else:
                print('Function type should be lr or l2 !!')
                exit(2)

def init():
    try:
        opts, args = getopt.getopt(sys.argv[1:],"rn:f:")
    except getopt.GetoptError as err:
        print(str(err))
        # print usage
        usage()
        exit(2)
    opt_parse(opts)
    #print(fun_type, pool_num, run_opt)

def exec_cmd(cmd, idx=-1, run=False):
    print('RUN #{:2}  {}'.format(idx,cmd))
    if run: os.system(cmd)
    print('Done#{:2}  {}'.format(idx,cmd))

def run_data(args):
    fpath = DATA_DIR.format(args['fname'])
    args['fpath'] = fpath
    binary_path = os.path.realpath(BINARY_PATH.format(args=args))
    if not os.path.exists(binary_path):
        print('Binary file not found. You need to build the codes first.')
        return
    if not os.path.exists(fpath):
        print(fpath,'not found.')
        return

    if not os.path.exists(LOG_P):
         print('create dir', LOG_P)
         os.system('mkdir -p {}'.format(LOG_P))

    if not os.path.exists(MODEL_P):
         print('create dir', MODEL_P)
         os.system('mkdir -p {}'.format(MODEL_P))

    log_name = LOG_DIR.format(args=args)

    if args['tp'][-2:] == '_m':
        cmd = run_multi_cmd.format(args=args, log=log_name)
    else:
        cmd = run_cmd.format(args=args, log=log_name, model_dir=MODEL_P)
    #pprint(cmd)
    exec_cmd(cmd, idx=args['idx'], run=run_opt)

if __name__ == '__main__':
    init()

    print('Total #experiments: {}'.format(len(tps)*len(cs)*len(datasets)))
    count = 0
    p =  Pool(pool_num)

    for data in datasets:
        fpath = DATA_DIR.format(data)
        if not os.path.exists(fpath):
            os.system('wget https://www.csie.ntu.edu.tw/~cjlin/libsvmtools/datasets/binary/{}.bz2'.format(data))
            os.system('mv {}.bz2 {}'.format(data, DATA_DIR.format('')))
            os.system('bzip2 -d {}.bz2'.format(DATA_DIR.format(data)))

    for data in datasets:
        args = []
        for c in cs:
            for e in eps:
                for tp in tps:
                    count += 1
                    #arg = dict(arg_tmp)
                    arg = dict()
                    arg['tp'] = tp
                    arg['c_name'] = c
                    if USE_BEST:
                        arg['c'] = best_c[fun_type][data] * c
                    else:
                        arg['c'] = c
                    arg['fname'] = data
                    arg['e'] = e
                    if arg['fname'] in eps_exception and tp not in eps_ref:
                        arg['e'] = eps_exception[arg['fname']]
                    arg['idx'] = count

                    arg['s'] = ARG_DIC[tp][0]
                    arg['m'] = ARG_DIC[tp][1]

                    args.append(arg)
        p.map(run_data, args)
